@@ -11,47 +11,21 @@
 
 namespace torch::cuda::CUDAPluggableAllocator {
 
-using MallocFuncType = void*(size_t, int, hipStream_t);
-using FreeFuncType = void(void*, size_t, int, hipStream_t);
-
-// A CUDAPluggableAllocatorDeleterContext object is used as the `ctx`
-// argument for DataPtr. We need context because a user can use
-// multiple allocators in the same PyTorch program, and
-// the allocators can have different free functions, such as:
-// free, hipFree, cudaFreeAsync, ncclMemFree etc.
-struct TORCH_CUDA_CPP_API CUDAPluggableAllocatorDeleterContext {
-  explicit CUDAPluggableAllocatorDeleterContext(
-      std::function<FreeFuncType> free_fn,
-      void* data,
-      size_t size,
-      int device,
-      hipStream_t stream);
-
-  void free();
-
- private:
-  std::function<FreeFuncType> free_fn_;
-  void* data_;
-  size_t size_;
-  int device_;
-  hipStream_t stream_;
-};
-
-#if defined(TORCH_HIP_VERSION)
+#if defined(USE_ROCM)
 using streamType = c10::hip::HIPStream;
 #else
 using streamType = c10::hip::HIPStreamMasqueradingAsCUDA;
 #endif
 
-TORCH_CUDA_CPP_API std::shared_ptr<
+TORCH_HIP_CPP_API std::shared_ptr<
     c10::hip::HIPCachingAllocator::HIPAllocator>
 getCurrentAllocator();
-TORCH_CUDA_CPP_API std::shared_ptr<
+TORCH_HIP_CPP_API std::shared_ptr<
     c10::hip::HIPCachingAllocator::HIPAllocator>
 createCustomAllocator(
-    std::function<MallocFuncType> alloc_fn,
-    std::function<FreeFuncType> free_fn);
-TORCH_CUDA_CPP_API void changeCurrentAllocator(
+    std::function<void*(size_t, int, hipStream_t)> alloc_fn,
+    std::function<void(void*, size_t, int, hipStream_t)> free_fn);
+TORCH_HIP_CPP_API void changeCurrentAllocator(
     const std::shared_ptr<c10::hip::HIPCachingAllocator::HIPAllocator>&
         allocator);
 
@@ -63,17 +37,21 @@ struct _AllocationMetadata {
       hipStream_t stream);
   size_t size;
   c10::DeviceIndex device_idx;
-  hipStream_t stream;
+  hipStream_t stream{};
 };
 
-struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
+struct TORCH_HIP_CPP_API CUDAPluggableAllocator
     : public c10::hip::HIPCachingAllocator::HIPAllocator {
   CUDAPluggableAllocator(
-      std::function<MallocFuncType> alloc_fn,
-      std::function<FreeFuncType> free_fn);
+      std::function<void*(size_t, int, hipStream_t)> alloc_fn,
+      std::function<void(void*, size_t, int, hipStream_t)> free_fn);
 
   CUDAPluggableAllocator(CUDAPluggableAllocator& other);
-  CUDAPluggableAllocator& operator=(CUDAPluggableAllocator& other) = delete;
+  CUDAPluggableAllocator(CUDAPluggableAllocator&& other) = delete;
+  CUDAPluggableAllocator& operator=(const CUDAPluggableAllocator& other) =
+      delete;
+  CUDAPluggableAllocator& operator=(CUDAPluggableAllocator&& other) = delete;
+  ~CUDAPluggableAllocator() override = default;
 
   void set_init_fn(std::function<void(int)> init_fn);
 
@@ -108,8 +86,13 @@ struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
   void raw_delete(void* ptr) override;
   void init(int device_count) override;
   bool initialized() override;
+  double getMemoryFraction(c10::DeviceIndex device) override;
   void setMemoryFraction(double fraction, c10::DeviceIndex device) override;
-  void emptyCache() override;
+  void emptyCache(c10::hip::MempoolId_t mempool_id = {0, 0}) override;
+  void enable(bool) override {}
+  bool isEnabled() const override {
+    return true;
+  }
   void cacheInfo(c10::DeviceIndex device, size_t* largestBlock) override;
   void* getBaseAllocation(void* ptr, size_t* size) override;
 
@@ -119,7 +102,8 @@ struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
       c10::DeviceIndex device) override;
   void resetAccumulatedStats(c10::DeviceIndex device) override;
   void resetPeakStats(c10::DeviceIndex device) override;
-  c10::hip::HIPCachingAllocator::SnapshotInfo snapshot() override;
+  c10::hip::HIPCachingAllocator::SnapshotInfo snapshot(
+      c10::hip::MempoolId_t mempool) override;
   void beginAllocateToPool(
       c10::DeviceIndex device,
       c10::hip::MempoolId_t mempool_id,
@@ -136,7 +120,8 @@ struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
       bool enabled,
       c10::hip::HIPCachingAllocator::CreateContextFn context_recorder,
       size_t alloc_trace_max_entries,
-      c10::hip::HIPCachingAllocator::RecordContext when) override;
+      c10::hip::HIPCachingAllocator::RecordContext when,
+      bool clearHistory) override;
   void attachOutOfMemoryObserver(
       c10::hip::HIPCachingAllocator::OutOfMemoryObserver observer) override;
   void attachAllocatorTraceTracker(
@@ -162,8 +147,8 @@ struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
   void copy_data(void* dest, const void* src, std::size_t count) const final;
 
  protected:
-  std::function<MallocFuncType> alloc_fn_;
-  std::function<FreeFuncType> free_fn_;
+  std::function<void*(size_t, int, hipStream_t)> alloc_fn_;
+  std::function<void(void*, size_t, int, hipStream_t)> free_fn_;
   std::function<void(int)> init_fn_;
   std::function<void()> reset_fn_;
   std::function<void(double, int)> memory_fraction_fn_;
@@ -175,7 +160,7 @@ struct TORCH_CUDA_CPP_API CUDAPluggableAllocator
   std::function<void(int, c10::hip::MempoolId_t)> end_allocate_to_pool_fn_;
   std::function<void(int, c10::hip::MempoolId_t)> relase_pool_fn_;
   std::mutex allocator_mutex_;
-  // We do the bookeeping here in order to simplify custom allocators
+  // We do the bookkeeping here in order to simplify custom allocators
   std::unordered_map<void*, _AllocationMetadata> allocation_metadata_;
 
   bool initialized_ = false;
